#include "BitmapFormatSelector.h"
#include <TranslatorRoster.h>
//#include <FilePanel.h>
//#include <TranslationDefs.h>

const uint32 SAVE_FILE_PANEL_FORMAT = 'sfpf';
const uint32 SAVE_FILE_PANEL_SETTINGS = 'sfps';

TranslatorMenuItem::TranslatorMenuItem(const char *name, BMessage *message,
	translator_id id, uint32 format) : BMenuItem(name, message) {

	this->id = id;
	this->format = format;
}

BitmapFormatSelector::BitmapFormatSelector(BRect frame, const char *name,
	const char *label)
	: BView(frame, name, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW)
{
	BuildMenu();
	fFormatMenuField = new BMenuField(this->Bounds(), "MenuField",
		label, fFormatPopUpMenu);
	this->AddChild(fFormatMenuField);
}

void BitmapFormatSelector::BuildMenu()
{
	fFormatPopUpMenu = new BPopUpMenu("FormatPopUpMenu");
	BTranslatorRoster *roster = BTranslatorRoster::Default();
	translator_id *translators;
	int32 count;

	// Find all translators on the system
	roster->GetAllTranslators(&translators, &count);
	const translation_format *format;
	int32 format_count;
	
	for (int x = 0; x < count; x++) {
		// Determine which formats this one can write
		roster->GetOutputFormats(translators[x], &format, &format_count);
		for (int y = 0; y < format_count; y++) {
			// Check if this is an image translator
			if (format[y].group == B_TRANSLATOR_BITMAP) {
				// If this format saves to some native format build a menu item for it
				if (format[y].type == B_TRANSLATOR_BITMAP) continue;
				TranslatorMenuItem *item = new TranslatorMenuItem(format[y].name,
					new BMessage(SAVE_FILE_PANEL_FORMAT), translators[x], format[y].type);
				item->SetTarget(this);
				fFormatPopUpMenu->AddItem(item);
				break;
			}
		}
	}
	delete [] translators;
	
	// Pick the first translator in the list
	TranslatorMenuItem *item = (TranslatorMenuItem *)fFormatPopUpMenu->ItemAt(0);
	if (item != NULL) item->SetMarked(true);
}

TranslatorMenuItem *BitmapFormatSelector::GetCurrentMenuItem() {
	return (TranslatorMenuItem *)fFormatPopUpMenu->FindMarked();
}
