#include "FileView.h"
#include "MFileUtils.h"
#include "MToolTipStringView.h"
#include <Path.h>
#include <Window.h>

FileView::FileView(BRect frame, const char *name, const char *label,
	const char *text, const char *type, BMessage *message)
	: BView(frame, name, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW),
		fType(type)
{
	fMessage = message;
	//フォントサイズによってビューの高さを調整する
	font_height height;
	this->GetFontHeight(&height);
	float totalFontHeight = height.ascent + height.descent;
	//ボックスとテキストとのマージン
	//本当はボックスの枠の太さによって変えたほうがいいかもしれない
	float margin = 3;
	this->ResizeTo(this->Bounds().Width(), totalFontHeight*2 + margin*2);
	BRect boxFrame = this->Bounds();
	//ボックスで枠線をつける
	fBox = new BBox(boxFrame, "box", B_FOLLOW_ALL_SIDES);
	fBox->SetLabel(label);
	
	BRect stringFrame = fBox->Bounds();
	stringFrame.top += totalFontHeight;
	stringFrame.InsetBy(margin, margin);
	//ファイル名を表示するビュー
	fString = new MToolTipStringView(stringFrame, "file", text, B_FOLLOW_ALL_SIDES);
	
	fBox->AddChild(fString);
	this->AddChild(fBox);
}

void FileView::MessageReceived(BMessage *message)
{
	int32 i = 0;
	entry_ref ref;
	//refを受け取ったらパスを表示
	while (message->FindRef("refs", i, &ref) == B_OK)
	{
		if (MFileUtils::IsThisType(&ref, fType.String()) == true)
		{
			BPath path(&ref);
			fString->SetText(path.Path());
			if (fMessage != NULL)
				this->Window()->PostMessage(fMessage);
			break;
		}
		i++;
	}
	
}

const char *FileView::Path() const
{
	return fString->Text();
}

