#include "MIWindow.h"
#include <ScrollView.h>
#include <Button.h>
#include <File.h>
#include <Path.h>
#include <Alert.h>
#include <Entry.h>
#include <TranslatorRoster.h>
#include <BitmapStream.h>
#include <stdio.h>
#include <stdlib.h>
#include <Font.h>

const int32 kListViewHeight = 100;
const int32 kMargin = 3;

const uint32 TOGGLE_OPTION = 'tgop';
const uint32 START_PROCESS = 'stpr';
const uint32 VIDEO_SELECTED = 'vdeo';
const uint32 PREFIX_CHANGED = 'prch';
const uint32 SUFFIX_CHANGED = 'poch';
const uint32 INIT_NUM_CHANGED = 'inch';
const uint32 EVERY_CHANGED = 'evch';

MIWindow::MIWindow()
	: BWindow(BRect(100, 100, 300, 400), "Movie2Image",
		B_TITLED_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL,
		/*B_NOT_H_RESIZABLE |*/ B_ASYNCHRONOUS_CONTROLS | B_QUIT_ON_WINDOW_CLOSE)
{
	fVideoCutter = NULL;
	InitContent();
}

void MIWindow::InitContent()
{
	//ベース
	//ウィンドウいっぱいに
	BRect baseFrame = this->Bounds();
	BView *base = new BView(baseFrame, "base_view",
		B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	this->AddChild(base);
	//メインコントロール
	//とりあえずウィンドウいっぱい
	BRect mainFrame = base->Bounds();
	mainFrame.InsetBy(kMargin, kMargin);
	BView *main = new BView(mainFrame, "main_view",
		B_FOLLOW_LEFT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW);
	base->AddChild(main);
	//オプションコントロール
	//ウィンドウの右側に隠す
	BRect optionFrame = base->Bounds();
	optionFrame.OffsetBy(optionFrame.Width() + 1, 0);
	optionFrame.InsetBy(kMargin, kMargin);
	BView *option = new BView(optionFrame, "option_view",
		B_FOLLOW_LEFT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW);
	base->AddChild(option);
	//ビデオ指定ビュー
	BRect fileFrame = main->Bounds();
	fVideoSpecifier = new FileView(fileFrame, "video_view", "Movie File",
		"Drop Movie File Here", "video", new BMessage(VIDEO_SELECTED));
	main->AddChild(fVideoSpecifier);
	//フォルダ指定ビュー
	BRect dirFrame = main->Bounds();
	dirFrame.top = fVideoSpecifier->Frame().bottom + 1 + kMargin;
	fFolderSpecifier = new FileView(dirFrame, "dir_view", "Save Here",
		"Drop Folder To Save", "application/x-vnd.Be-directory");
	main->AddChild(fFolderSpecifier);
	//ファイルリストビュー
	BRect listFrame = main->Bounds();
	listFrame.top = fFolderSpecifier->Frame().bottom + 1 + kMargin;
	listFrame.right -= B_V_SCROLL_BAR_WIDTH;
	listFrame.InsetBy(kMargin, 0);
	listFrame.bottom = listFrame.top + kListViewHeight;
	fFileList = new FileNameListView(listFrame, "list");
	BScrollView *scroll = new BScrollView("scroll_list", fFileList,
		B_FOLLOW_LEFT | B_FOLLOW_TOP_BOTTOM, 0, false, true);
	main->AddChild(scroll);

	//開始ボタン
	float width, height;
	BRect doFrame = main->Bounds();
	doFrame.top = scroll->Frame().bottom + 1 + kMargin;
	BButton *doButton = new BButton(doFrame, "doit", "Do It!",
		new BMessage(START_PROCESS),B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	main->AddChild(doButton);
	doButton->GetPreferredSize(&width, &height);
	doButton->ResizeTo(width, height);
	//オプションボタン
	/*
	BRect optionOpenFrame = main->Bounds();
	optionOpenFrame.top = scroll->Frame().bottom + 1 + kMargin;
	BButton *optionButton = new BButton(optionOpenFrame, "option", ">",
		new BMessage(TOGGLE_OPTION), B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM);
	main->AddChild(optionButton);
	optionButton->GetPreferredSize(&width, &height);
	optionButton->ResizeTo(width, height);
	optionButton->MoveTo(optionOpenFrame.right - width, optionOpenFrame.top);
	*/
	//baseビューのサイズ調整
	//base->ResizeTo(base->Bounds().Width(), optionButton->Frame().bottom);
	//ウィンドウサイズの調整
	/*
	float windowHeight;
	BPoint windowLeftTop = this->LeftTop();
	BPoint buttonLeftBottom;
	buttonLeftBottom.y = doButton->ConvertToScreen(doButton.LeftTop());
	windowHeight = 
	this->ResizeTo(this->Bounds().Width(), windowHeight);
	*/
	//オプションの中身
	//フォントサイズによってビューの高さを調整する
	font_height fontHeight;
	option->GetFontHeight(&fontHeight);
	float totalFontHeight = fontHeight.ascent + fontHeight.descent;
	BRect miniFrame = option->Bounds();
	miniFrame.bottom = miniFrame.top + totalFontHeight*2;
	//フォーマットセレクタ
	fFormatSelector = new BitmapFormatSelector(miniFrame,
		"selector", "Format");
	option->AddChild(fFormatSelector);

	//プレフィクス
	miniFrame.OffsetBy(0, miniFrame.Height() + 1);
	fPrefix = new BTextControl(miniFrame,
		"prefix", "Prefix", "", new BMessage(PREFIX_CHANGED));
	option->AddChild(fPrefix);
	//suffix
	miniFrame.OffsetBy(0, miniFrame.Height() + 1);
	fSuffix = new BTextControl(miniFrame,
		"suffix", "Suffix", "", new BMessage(SUFFIX_CHANGED));
	option->AddChild(fSuffix);
	//initial number
	miniFrame.OffsetBy(0, miniFrame.Height() + 1);
	fInitialNumber = new BTextControl(miniFrame, "initial_number",
		"Initial Number", "0", new BMessage(INIT_NUM_CHANGED));
	for (int x = 0; x < 256; x++) {
		if (x < '0' || x > '9') fInitialNumber->TextView()->DisallowChar(x);
	}
	option->AddChild(fInitialNumber);
	//every x frame
	miniFrame.OffsetBy(0, miniFrame.Height() + 1);
	fEveryXFrame = new BTextControl(miniFrame, "every",
		"Every x frame", "1", new BMessage(EVERY_CHANGED));
	for (int x = 0; x < 256; x++) {
		if (x < '0' || x > '9') fEveryXFrame->TextView()->DisallowChar(x);
	}
	option->AddChild(fEveryXFrame);
	//画像プレビュー
	miniFrame.OffsetBy(0, miniFrame.Height() + 1);
	BRect imageFrame = miniFrame;
	imageFrame.bottom = imageFrame.top + imageFrame.Width();
	fImageView = new BBox(imageFrame, "image",
		B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
	option->AddChild(fImageView);
	return;
}

void MIWindow::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
		case TOGGLE_OPTION:
		{
			break;
		}
		case START_PROCESS:
		{
			CreateBitmap();
			break;
		}
		case VIDEO_SELECTED:
		{
			RefreshVideoInfo();
			break;
		}
		case PREFIX_CHANGED:
		{
			fFileList->SetPrefix(fPrefix->Text());
			break;
		}
		case SUFFIX_CHANGED:
		{
			fFileList->SetSuffix(fSuffix->Text());
			break;
		}
		case INIT_NUM_CHANGED:
		{
			int32 init = atol(fInitialNumber->Text());
			if (init >= 0)
				fFileList->SetInitialNumber(init);
			break;
		}
		case EVERY_CHANGED:
		{
			fFileList->DeselectAll();
			int32 x = atol(fEveryXFrame->Text());
			if (x > 0)
			{
				int32 index = 0;
				int32 count = fFileList->CountItems();
				while(index <= count)
				{
					fFileList->Select(index, true);
					index += x;
				}
			}
			break;
		}
		case FILE_LIST_SELECTION_CHANGED:
		{
			//(new BAlert("", "FILE_LIST_SELECTION_CHANGED", "OK"))->Go();
			if (CheckVideo() != B_OK) return;
			int32 selected = fFileList->CurrentSelection(0);
			if (selected <= 0) return;//Every x 対策。ひどい仕様だ。
			BBitmap *bitmap = fVideoCutter->GetBitmap(selected);
			if (CheckBitmap(bitmap) != B_OK)
			{
				(new BAlert("", "Invalid Bitmap", "OK"))->Go();
				return;
			}
			fImageView->SetViewBitmap(bitmap, bitmap->Bounds(),
				fImageView->Bounds());
			fImageView->Invalidate();
			delete bitmap;
			break;
		}
		default :
		{
			BWindow::MessageReceived(message);
			break;
		}
	}
}

void MIWindow::CreateBitmap()
{
	status_t sts;
	
	CheckVideo();
	
	int32 selected;
	BStringItem *item;
	BFile file;
	BPath path;
	BPath folder(fFolderSpecifier->Path());
	sts = folder.InitCheck();
	if (sts != B_OK) (new BAlert("", "Folder InitCheck Failed", "OK"))->Go();
	TranslatorMenuItem *formatItem = fFormatSelector->GetCurrentMenuItem();
	uint32 format = formatItem->format;
	translator_id id = formatItem->id;
	int32 i = 0;
	while ((selected = fFileList->CurrentSelection(i)) >=0)
	{
		i++;
		item = (BStringItem*)fFileList->ItemAt(selected);
		path = folder;
		path.Append(item->Text());
		sts = file.SetTo(path.Path(), B_WRITE_ONLY | B_CREATE_FILE);
		if (sts != B_OK)
			(new BAlert("", "File Creation Failed", "OK"))->Go();
		BTranslatorRoster *roster = BTranslatorRoster::Default(); 
		BBitmap *bitmap = fVideoCutter->GetBitmap(selected);
		if (CheckBitmap(bitmap) != B_OK)
		{
			(new BAlert("", "Invalid Bitmap", "OK"))->Go();
			continue;
		}
		BBitmapStream stream(bitmap);
		roster->Translate(id, &stream, NULL, &file, format);
		file.Sync();
		stream.DetachBitmap(&bitmap);
		delete bitmap;
	}
	
}

status_t MIWindow::CheckBitmap(BBitmap *bitmap)
{
	if (bitmap == NULL)
	{
		return B_ERROR;
	}
	if (bitmap->IsValid() == false)
	{
		return B_ERROR;
	}
	return B_OK;
}

status_t MIWindow::CheckVideo()
{
	if (fVideoCutter == NULL)
	{
		(new BAlert("", "Please select valid movie", "OK"))->Go();
		return B_ERROR;
	}
	status_t sts = fVideoCutter->InitCheck();
	if (sts != B_OK)
	{
		(new BAlert("", "Invalid Movie File ... maybe", "OK"))->Go();
		return sts;
	}
	return B_OK;
}

void MIWindow::RefreshVideoInfo()
{
	delete fVideoCutter;
	entry_ref ref;
	status_t sts;
	sts = get_ref_for_path(fVideoSpecifier->Path(), &ref);
	if (sts != B_OK)
	{
		(new BAlert("", "Couldn't get path for movie", "OK"))->Go();
		return;
	}
	fVideoCutter = new MovieDecomposer(ref);
	if (CheckVideo() != B_OK) return;
	int64 frame = fVideoCutter->CountFrames();
	fFileList->SetTotalNumber(frame);
}
