
#include <stdlib.h>
#include <string.h>

#include <Application.h>

#include "MToolTipStringView.h"
#include "TToolTip.h"

MToolTipStringView::MToolTipStringView(BRect rect,
							   const char *name,
							   const char *text,
							   uint32 resize_mask,
							   uint32 flags)
			   :BStringView(rect, name, text, resize_mask, flags),
			    fIn(false)
{
	fTip = "";
}

//--------------------------------------------------------------------

MToolTipStringView::~MToolTipStringView()
{
	// kill off any current ToolTip
	if (fIn)
		be_app->PostMessage(eToolTipStop);
	// free cached ToolTip
}

//--------------------------------------------------------------------

void MToolTipStringView::MouseMoved(BPoint where, uint32 code, const BMessage *msg)
{
	// if mouse has moved into our view, our window is active and it previously
	// wasn't in, send a message to start the ToolTip
	if ((Bounds().Contains(where)) && (Window()->IsActive())) {
		if (!fIn) {
			BFont font;
			this->GetFont(&font);
			float stringWidth = font.StringWidth(this->Text());
			float viewWidth = this->Bounds().Width();
			if (stringWidth > viewWidth)
			{
				BMessage	msg(eToolTipStart);
	
				msg.AddPoint("start", ConvertToScreen(where));
				msg.AddRect("bounds", ConvertToScreen(Bounds()));
				msg.AddString("string", this->Text());
				be_app->PostMessage(&msg);
				fIn = true;
			}
		}
	}
	// otherwise stop the message
	else if (fIn) {
		be_app->PostMessage(eToolTipStop);
		fIn = false;
	}
	BStringView::MouseMoved(where, code, msg);
}
