#include "MovieDecomposer.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <Rect.h>
#include <Entry.h>

#include <MediaFile.h>
#include <MediaTrack.h>

// this small sample code shows how to extract video frames and audio
// samples from a file.


// ProcessBitmap() is called on every decoded video frame. Useful code could be
// added here.

BBitmap *MovieDecomposer::GetBitmap(int64 frame)
{
	// iterate through all the frames and call the processing function
	status_t err;
	BRect bounds(0.0, 0.0, fFormat.u.encoded_video.output.display.line_width - 1.0, 
			   fFormat.u.encoded_video.output.display.line_count - 1.0);
	BBitmap *bitmap = new BBitmap(bounds, B_RGB32);

	// specifiy the decoded format. we derive this information from
	// the encoded format.

	memset(&fFormat, 0, sizeof(media_format));
	fFormat.u.raw_video.last_active = (int32) (bounds.Height() - 1.0);
	fFormat.u.raw_video.orientation = B_VIDEO_TOP_LEFT_RIGHT;
	fFormat.u.raw_video.pixel_width_aspect = 1;
	fFormat.u.raw_video.pixel_height_aspect = 3;
	fFormat.u.raw_video.display.format = bitmap->ColorSpace();
	fFormat.u.raw_video.display.line_width = (int32) bounds.Width();
	fFormat.u.raw_video.display.line_count = (int32) bounds.Height();
	fFormat.u.raw_video.display.bytes_per_row = bitmap->BytesPerRow();
	err = fTrack->DecodedFormat(&fFormat);
	if (err) {
		printf("error with BMediaTrack::DecodedFormat() -- %s\n", strerror(err));
		delete bitmap;
		return NULL;
	}
		
	char			*buffer;
	buffer = (char *)bitmap->Bits();
	media_header mh;
	int64			frameCount;
	
	err = fTrack->SeekToFrame(&frame);
	if (err) {
		printf("BMediaTrack::SeekToFrame error -- %s\n", strerror(err));
		delete bitmap;
		return NULL;
	}
	err = fTrack->ReadFrames(buffer, &frameCount, &mh);
	if (err) {
		printf("BMediaTrack::ReadFrames error -- %s\n", strerror(err));
		delete bitmap;
		return NULL;
	}
	return bitmap;
}


MovieDecomposer::MovieDecomposer(entry_ref ref)
{
	fMediaFile = new BMediaFile(&ref);
	fCheck = Init();
}

MovieDecomposer::~MovieDecomposer()
{
	fMediaFile->ReleaseTrack(fTrack);
	delete fMediaFile;
}

int64 MovieDecomposer::CountFrames()
{
	return fTrack->CountFrames();
}

status_t MovieDecomposer::InitCheck()
{
	return fCheck;
}

status_t MovieDecomposer::Init()
{
	status_t		err;
	int32			i;
	int32 numTracks;

	// instantiate a BMediaFile object, and make sure there was no error.
	if (fMediaFile == NULL) return B_ERROR;
	err = fMediaFile->InitCheck();
	if (err) {
		printf("cannot contruct BMediaFile object -- %s\n", strerror(err));
		return err;
	}
	
	// count the tracks and instanciate them, one at a time

	numTracks = fMediaFile->CountTracks();

	for (i = 0; i < numTracks; i++) {
		fTrack = fMediaFile->TrackAt(i);
		if (!fTrack) {
			printf("cannot contruct BMediaTrack object\n");
			return B_ERROR;
		}

		// get the encoded format

		err = fTrack->EncodedFormat(&fFormat);
		if (err) {
			printf("BMediaTrack::EncodedFormat error -- %s\n", strerror(err));
			return err;
		}

		switch(fFormat.type)
		{
	
			// handle the case of an encoded video track
	
		 	case B_MEDIA_ENCODED_VIDEO:
			{
				return B_OK;
				break;
			}
			// handle the case of a raw audio track
	
			case B_MEDIA_RAW_AUDIO:
			default:
				fMediaFile->ReleaseTrack(fTrack);
				break;
		}

	}
	return B_ERROR;
}